# !/usr/bin/python
# -*- coding: utf-8 -*-

# Glitter Toolkit

__authors__ = ["Jan Jokela <janjokela@gmail.com>"]
__licenses__ = ["LICENSE.LGPL"]
__description__ = "Container widget"

from widget import Widget

POLICY_NONE = 'POLICY_NONE'
POLICY_EXPAND = 'POLICY_EXPAND'
POLICY_MINIMUM = 'POLICY_MINIMUM'

class Container(Widget):
    """
    A container widget provides a representation of a graphical  container 
    that not only is intended as a placeholder for children but also as a base 
    class for other container-type widgets.
    
    It offers powerfull layout capabilities through the usage of minimum and 
    natural sizes and positions. It also features resolution independence 
    through the use of relative values, millimeters and clutter units.
    
    """ 
    
    def __init__(self):
        """ Initialize the container """
        
        super(Container, self).__init__()
        
        self._natural_width = 0.0
        self._natural_height = 0.0
        self._minimum_width = 0.0
        self._minimum_height = 0.0
        self._natural_x = 0.0
        self._natural_y = 0.0
        self._minimum_x = 0
        self._minimum_y = 0
        self._h_offset = 0.0
        self._v_offset = 0.0
        
        self._size_ratio = 0.0
        
    def _update_style(self, props=None):
        """ Update style """
        
        super(Container, self)._update_style(props)
        
        if not props:
            return
        
        for key, value in props:
            if key == 'natural-width':
                self.natural_width = value
            elif key == 'natural-height':
                self.natural_height = value
            elif key == 'minimum-width':
                self.minimum_width = value
            elif key == 'minimum-height':
                self.minimum_height = value
            elif key == 'natural-x': 
                self.natural_x = value
            elif key == 'natural-y':
                self.natural_y = value
            elif key == 'minimum-x':
                self.minimum_x = value
            elif key == 'minimum-y':
                self.minimum_y = value
            elif key == 'h-offset':
                self.h_offset = value
            elif key == 'v-offset':
                self.v_offset = value
                
    def _update_layout(self):
        """ Updates layout """
        
        if self.get_parent():
        
            # Natural an minimum sizes
            n_width = int(self.natural_width * self.get_parent().get_widthu())
            n_height = int(self.natural_height * self.get_parent().get_heightu())
            
            m_width = self.minimum_width
            m_height = self.minimum_height
            
            # Preferred size is the natural one, except if lower than minimum
            if n_width >= m_width:
                tmp_width = n_width
            else:
                tmp_width = m_width
            if n_height >= m_height:
                tmp_height = n_height
            else:
                tmp_height = m_height
                
            # Account for size ratios
            if self.size_ratio > 0:
                tmp_width = int(self.size_ratio * tmp_height)
            elif self.size_ratio < 0:
                tmp_height = int(-self.size_ratio * tmp_width)
            
            # Set size
            self.set_widthu(int(tmp_width))
            self.set_heightu(int(tmp_height))

            # Natural and minimum positions
            n_x = self.natural_x * self.get_parent().get_widthu()
            n_y = self.natural_y * self.get_parent().get_heightu()
            
            m_x = self.minimum_x
            m_y = self.minimum_y
            
            if n_x >= m_x:
                tmp_x = n_x
            else:
                tmp_x = m_x
            if n_y >= m_y:
                tmp_y = n_y
            else:
                tmp_y = m_y
            
            tmp_x = int(tmp_x)
            tmp_y = int(tmp_y)
            
            #print 'tmp_x is: ' + str(self.natural_x)
            self.set_positionu(tmp_x, tmp_y)
        
    def get_natural_width(self):
        """ Retrieve natural width """
        
        return self._natural_width
        
    def set_natural_width(self, value):
        """ Sets natural width
        
        value -- (float) Width as a factor of parent width
        """
    
        self._natural_width = value
      
    natural_width = property(get_natural_width, set_natural_width)
    
    def get_natural_height(self):
        """ Retrieve natural height """
        
        return self._natural_height

    def set_natural_height(self, value):
        """ Sets natural height
        
        value -- (float) Height as a factor of parent height
        """
        
        self._natural_height = value
        
    natural_height = property(get_natural_height, set_natural_height)

    def get_minimum_width(self):
        """ Retrieve minimum width """
    
        return self._minimum_width
        
    def set_minimum_width(self, value):
        """ Sets minimum width 
        
        value -- (int) Minimum width in clutter units
        """
    
        self._minimum_width = value
        
    minimum_width = property(get_minimum_width, set_minimum_width)
        
    def get_minimum_height(self):
        """ Retrieve minimum height """
    
        return self._minimum_height
        
    def set_minimum_height(self, value):
        """ Sets minimum height 
        
        value -- (int) Minimum height in clutter units
        """
    
        self._minimum_height = value
        
    minimum_height = property(get_minimum_height, set_minimum_height)

    def get_size_ratio(self):
        """ Retrieve size ratio """
        
        return self._size_ratio
        
    def set_size_ratio(self, value):
        """ Sets size ratio
        
        value -- (float) Size ratio. Positive values for width as a factor of 
                 height. Negative values for height as a factor of width.
        """
    
        self._size_ratio = value
        
    size_ratio = property(get_size_ratio, set_size_ratio)

    def get_width_policy(self):
        """ Retrieve width policy """
        
        return self._width_policy
        
    def set_width_policy(self, value):
        """ Sets width policy 
        
        value -- (str) NATURAL or MINIMUM width
        """
        
        self._width_policy = value
        
    width_policy = property(get_width_policy, set_width_policy)
    
    def get_height_policy(self):
        """ Retrieve height policy """
        
        return self._height_policy
        
    def set_height_policy(self, value):
        """ Sets height policy
        
        value -- (str) NATURAL or MINIMUM height
        """
        
        self._height_policy = value
        
    height_policy = property(get_height_policy, set_height_policy)    
    
    def get_natural_x(self):
        """ Retrieve minimum x """
        
        return self._natural_x
        
    def set_natural_x(self, value):
        """ Sets natural x
        
        value -- (float) X position as a factor of parent width
        """
        
        self._natural_x = value
    
    natural_x = property(get_natural_x, set_natural_x)
    
    def get_natural_y(self):
        """ Retrieve natural y """
        
        return self._natural_y
        
    def set_natural_y(self, value):
        """ Sets natural y
        
        value -- (float) Y as a factor of parent height
        """
        
        self._natural_y = value 
        
    natural_y = property(get_natural_y, set_natural_y)
    
    def get_minimum_x(self):
        """ Retrieve minimum y """
        
        return self._minimum_y
        
    def set_minimum_x(self, value):
        """ Sets minimum x
        
        value -- (float) Minimum x in clutter units
        """
        
        self._minimum_x = value
        
    minimum_x = property(get_minimum_x, set_minimum_x)
    
    def get_minimum_y(self):
        """ Retrieve minimum y """
        
        return self._minimum_y
        
    def set_minimum_y(self, value):
        """ Sets minimum y
        
        value -- (float) Minimum y in clutter units
        """
        
        self._minimum_y = value
        
    minimum_y = property(get_minimum_y, set_minimum_y)
    
    def get_h_offset(self):
        """ Retrieve horizontal offset """
        
        return self._h_offset
        
    def set_h_offset(self, value):
        """ Sets horizontal offset. Overwrites values for natural x and width. 
        
        value -- (float) offset as a factor of parent width
        """
        
        self._h_offset = value
        self.natural_x = value
        self.natural_width = 1.0 - 2 * value
        
    h_offset = property(get_h_offset, set_h_offset)
    
    def get_v_offset(self):
        """ Retrieve vertical offset """
        
        return self._v_offset
        
    def set_v_offset(self, value):
        """ Sets vertical offset. Overwrites values for natural y and height. 
        
        value -- (float) offset as a factor of parent height
        """

        self._v_offset = value
        self.natural_y = value
        if self.natural_height > 0: 
            self.natural_height = self.natural_height - 2 * value
        
    v_offset = property(get_v_offset, set_v_offset)

