# !/usr/bin/python
# -*- coding: utf-8 -*-

# Glitter Toolkit

__authors__ = ["Jan Jokela <janjokela@gmail.com>"]
__licenses__ = ["LICENSE.LGPL"]
__description__ = "Base class for widgets" 

import gobject
import clutter

from theme import Theme
from style import Style

STATE_NORMAL = 'STATE_NORMAL'
STATE_HIGHLIGHT = 'STATE_HIGHLIGHT'
STATE_ACTIVE = 'STATE_ACTIVE'
STATE_SELECTED = 'STATE_SELECTED'
STATE_INSENSITIVE = 'STATE_INSENSITIVE'

class Widget(clutter.Group):
    """
    Base class for Glitter widgets.
    
    A widget inherits from a clutter group actor, which means each and every 
    widget can have children added or removed, working as a scenegraph. 
    Moreover, all the remaining underlying clutter API is exposed.
    
    The widget object also holds widget specific properties like current state 
    and styles.
    
    """

    # Signals    
    __gsignals__ = {
        'state-changed': (gobject.SIGNAL_RUN_LAST,
                          gobject.TYPE_BOOLEAN,
                          (gobject.TYPE_PYOBJECT,)),
        'style-changed': (gobject.SIGNAL_RUN_LAST,
                          gobject.TYPE_BOOLEAN,
                          (gobject.TYPE_PYOBJECT,))
    }
    
    def __init__(self):
        """ Initialize widget """
        
        super(Widget, self).__init__()
        
        self._state = STATE_NORMAL
        self._style = None
        self._styles = {}
        
        self._init_style()
        
    def _init_style(self):
        """ Initialize widget style """
                
        theme = Theme.get_default()
                  
        self._styles = {}
                     
        # Widget styles are a merge of the respective state styles for the 
        # widget and all its subclasses             
        base = self.__class__
        while issubclass(base, Widget): 
            module_name = base.__name__
            for state in (STATE_NORMAL, STATE_HIGHLIGHT, STATE_ACTIVE, 
                          STATE_SELECTED, STATE_INSENSITIVE):
                style = theme.get_style(module_name, state)
                if state in self._styles and style:
                    self._styles[state] = self._styles[state].merge(style)
                elif style:
                    self._styles[state] = style
                    """if state != STATE_NORMAL:
                        self._styles[state] = self._styles[STATE_NORMAL].merge(style)
                    else:
                        self._styles[state] = style"""
            base = base.__base__ 
        
        self._style = self._styles[self.state]

    def _update_style(self, props=None):
        """ Metaclass for style updates """

    # Properties
    
    def get_state(self):
        """ Retrieve widget state """
        
        return self._state
        
    def set_state(self, state):
        """ Sets the state for the widget and emits a 'state-changed' signal 
        case necessary
        
        state -- enum(STATE_NORMAL, STATE_HIGHLIGHT, STATE_ACTIVE, 
                 STATE_SELECTED, STATE_INSENSITIVE)
        """
        
        if self._state != state:
            self._state = state
            self.emit('state-changed', state)
            
    state = property(get_state, set_state)
    
    def get_style(self):
        """ Retrieve widget style """
        
        return self._style
        
    def set_style(self, style):
        """ Sets the style for the widget and emits a 'style-changed' signal 
        case necessary
        
        style -- (glitter.Style) 
        """
        
        if self._style != style:
            self._style = style
            self.emit('style-changed', style)
            
    style = property(get_style, set_style)

    def do_state_changed(self, previous_state):
        """ Widget state changed """
        
        self.style = self._styles[self.state]
        
    def do_style_changed(self, previous_style):
        """ Widget style changed """
        
        self._update_style(self.style)
        
        
